@@ -136,7 +136,7 @@ CartParser = StringParser(
         (
             Named(
                 Repeating(
-                    Capturing(PositiveInteger),
+                    Capturing(Integer),
                     min=3, max=3,
                     prefix=Optional(Whitespace),
                     suffix=Whitespace
@@ -160,14 +160,25 @@ CartParser = StringParser(
 
 # raise Exception(CartParser.regex)
 
-def cartesian_coordinates_parser(strs):
-    strss = "\n\n".join(strs)
+label_pattern = RegexPattern([" ", Integer, " "])
+def cartesian_coordinates_parser(strs, label_pattern=label_pattern):
+    # strss = "\n\n".join(strs)
 
-    parse = CartParser.parse_all(strss)
+    # dummy atoms come and go...so we need to loop, annoyingly
+
+    int_data = [
+        np.array(label_pattern.findall(s)).astype(int).reshape(-1, 3)
+        for s in strs
+    ]
+
+    coords = [
+        np.array(Number.findall(s)).astype(float).reshape(-1, 3)
+        for s in strs
+    ]
 
     coords = (
-        parse["GaussianStuff", 0],
-        parse["Coordinates"].array
+        int_data,
+        coords
     )
 
     return coords
@@ -881,6 +892,59 @@ GaussianLogComponents["AIMDTrajectory"] = {
     "mode"     : mode
 }
 
+force_block_tags = (
+    'Forces (Hartrees/Bohr)',
+    'Cartesian Forces:'
+)
+
+ForceParser = StringParser(
+    Repeating(
+        (
+            Named(
+                Repeating(
+                    Capturing(Integer),
+                    min=2, max=2,
+                    prefix=Optional(Whitespace),
+                    suffix=Whitespace
+                ),
+                "GaussianStuff", handler=StringParser.array_handler(dtype=int)
+            ),
+            Named(
+                Repeating(
+                    Capturing(Number),
+                    min = 3,
+                    max = 3,
+                    prefix=Optional(Whitespace),
+                    joiner = Whitespace
+                ),
+                "Forces", handler=StringParser.array_handler(dtype=float)
+            )
+        ),
+        suffix = Optional(Newline)
+    )
+)
+
+def parse_force_list(strs):
+
+    strss = "\n\n".join(strs)
+
+    parse = ForceParser.parse_all(strss)
+
+    coords = (
+        parse["GaussianStuff", 0],
+        parse["Forces"].array
+    )
+
+    return coords
+
+mode = "List"
+GaussianLogComponents["Gradients"] = {
+    "tag_start": force_block_tags[0],
+    "tag_end"  : force_block_tags[1],
+    "parser"   : parse_force_list,
+    "mode"     : mode
+}
+
 
 def parse_hessian_list(hessias):
     return [parse_weird_mat(h) for h in hessias]
@@ -894,7 +958,64 @@ GaussianLogComponents["Hessians"] = {
 }
 
 
-tag_start = """Full mass-weighted force constant matrix:"""
+cubic_block_tags = (
+    'Final third derivatives:',
+    'Diagonal'
+)
+label_pattern = RegexPattern([" ", PositiveInteger, " "])
+def parse_cubic_mat(pars, label_pattern=label_pattern): # identical to X-matrix parser...
+    """Parses the Hessian matrix block and returns stuff --> huge pain in the ass function"""
+    import numpy as np
+
+    energies = np.array([x.replace("D", "E") for x in DNumberPattern.findall(pars)])
+    l = np.max(np.array(label_pattern.findall(pars)).astype(int))
+    n = int( (-1 + np.sqrt(1 + 8*l))/2 )
+    k = len(energies) // l
+    X = np.empty((n, n, k))
+    # gaussian returns the data as blocks of 5 columns in the lower-triangle, annoyingly,
+    # so we need to rearrange the indices so that they are sorted to make this work
+    i, j = np.tril_indices(n)
+    energies_taken = 0
+    blocks = int(np.ceil(k/5))
+    for b in range(blocks):
+        bs = b * 5
+        be = min((b+1) * 5, k)
+        e_new=energies_taken+l*(be-bs)
+        e = energies[energies_taken:e_new]
+        energies_taken=e_new
+        X[i, j, bs:be] = e.reshape(-1, be-bs)
+        X[j, i, bs:be] = e.reshape(-1, be-bs)
+
+    return X
+
+def parse_cubics_list(hessias):
+    return [parse_cubic_mat(h) for h in hessias]
+
+mode = "List"
+GaussianLogComponents["CubicDerivs"] = {
+    "tag_start": cubic_block_tags[0],
+    "tag_end"  : cubic_block_tags[1],
+    "parser"   : parse_cubics_list,
+    "mode"     : mode
+}
+
+quartic_block_tags = (
+    'nuclear 4th derivatives',
+    'Numerical'
+)
+def parse_quartics_list(hessias):
+    return [parse_cubic_mat(h) for h in hessias]
+
+mode = "List"
+GaussianLogComponents["QuarticDerivs"] = {
+    "tag_start": quartic_block_tags[0],
+    "tag_end"  : quartic_block_tags[1],
+    "parser"   : parse_quartics_list,
+    "mode"     : mode
+}
+
+
+tag_start = """Harmonic frequencies (cm**-1)"""
 tag_end = """
 
 """
@@ -960,6 +1081,228 @@ GaussianLogComponents["NormalModes"] = {
 #     "mode"     : mode
 # }
 
+tag_start = "Excitation energies and oscillator strengths:"
+tag_end = "SavETr:"
+
+def parse_excited_states(blocks:str):
+    states = []
+    if blocks is not None:
+        blocks = blocks.strip().split(":")[1:]
+        for b in blocks:
+            lines = b.split("\n")
+            info = lines[0]
+            bits = info.strip().rsplit(maxsplit=6)
+            if len(bits) < 7: break
+            symmetry, energy, _, _, _, fspec, sspec = bits
+            _, osc = fspec.split("=")
+            _, ov = sspec.split("=")
+
+            state = {
+                "Energy": float(energy),
+                "Overlap": float(ov),
+                "Symmetry": symmetry,
+                "Transitions": {}
+            }
+            for l in lines[1:]:
+                l = l.strip()
+                if '->' in l:
+                    start, end = l.split('->')
+                    end, energy = end.split()
+                    state['Transitions'][(int(start), int(end))] = float(energy)
+            states.append(state)
+            if b.endswith('SavETr'): break
+
+    return states
+
+mode = "Single"
+GaussianLogComponents["ExcitedStates"] = {
+    "tag_start": tag_start,
+    "tag_end": tag_end,
+    "parser": parse_excited_states,
+    "mode": mode
+}
+
+tag_start = "Zeta(I,J)"
+tag_end = "Num. of Coriolis"
+
+def parse_coriolis(blocks):
+    if blocks is not None:
+        blocks = blocks.rsplit("\n", 1)[0]
+        cors = np.loadtxt(io.StringIO(blocks), skiprows=1, usecols=[1, 2, 3])
+        ax = np.loadtxt(io.StringIO(blocks), skiprows=1, usecols=[0], dtype=str)
+        ar = np.zeros((len(ax), 1), dtype=float)
+        for i, a in enumerate(ax):
+            if a == 'y':
+                ar[i, 0] = 1
+            elif a == 'z':
+                ar[i, 0] = 2
+        return np.concatenate([ar, cors], axis=1)
+
+mode = "Single"
+GaussianLogComponents["CoriolisTerms"] = {
+    "tag_start": tag_start,
+    "tag_end": tag_end,
+    "parser": parse_coriolis,
+    "mode": mode
+}
+
+
+tag_start = "K(I,J)"
+tag_end = "Num. of 2nd derivatives"
+
+def parse_quadratics(blocks):
+    blocks = blocks.rsplit("\n", 1)[0]
+    return np.loadtxt(io.StringIO(blocks), skiprows=1, usecols=[0, 1, 2])
+
+mode = "Single"
+GaussianLogComponents["QuadraticTerms"] = {
+    "tag_start": tag_start,
+    "tag_end": tag_end,
+    "parser": parse_quadratics,
+    "mode": mode
+}
+
+tag_start = "K(I,J,K)"
+tag_end = "Num. of 3rd derivatives"
+
+def parse_cubics(blocks):
+    blocks = blocks.rsplit("\n", 1)[0]
+    return np.loadtxt(io.StringIO(blocks), skiprows=1, usecols=[0, 1, 2, 3])
+
+mode = "Single"
+GaussianLogComponents["CubicTerms"] = {
+    "tag_start": tag_start,
+    "tag_end": tag_end,
+    "parser": parse_cubics,
+    "mode": mode
+}
+
+tag_start = "K(I,J,K,L)"
+tag_end = "Num. of 4th derivatives"
+
+def parse_quartics(blocks):
+    blocks = blocks.rsplit("\n", 1)[0]
+    return np.loadtxt(io.StringIO(blocks), skiprows=1, usecols=[0, 1, 2, 3, 4])
+
+mode = "Single"
+GaussianLogComponents["QuarticTerms"] = {
+    "tag_start": tag_start,
+    "tag_end": tag_end,
+    "parser": parse_quartics,
+    "mode": mode
+}
+
+tag_start = "1\\"
+tag_end = FileStreamerTag(
+    ("\\\\@", "\\\n \\@", "\\\\\n @"),
+    # follow_ups = (""" -----""",)
+)
+
+def skip_report_header(stuff):
+    if stuff[0] in {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'}:
+        return skip_report_header(stuff[1:])
+    return stuff
+
+num_pattern = Alternatives([
+    Integer,
+    Number,
+    IntBaseNumber,
+    (Number, "e", Integer)
+])
+
+scan_spec = namedtuple('scan_spec', ['value', 'steps', 'amount'])
+def parse_reports(blocks,
+                  endline_pattern=RegexPattern([Newline, Whitespace]),
+                  num_pattern=num_pattern,
+                  numblock_pattern=Repeating(num_pattern, suffix=Optional(","))
+                  ):
+    bits = []
+    for b in blocks:
+        b = endline_pattern.remove(b)
+        res = {}
+        subblocks = b.split('\\\\')
+        info = subblocks[0]
+        stuff = info.split("\\")
+        stuff = skip_report_header(stuff)
+        if stuff[0].startswith('GINC-'):
+            res['node'] = stuff[0][5:]
+            stuff = stuff[1:]
+        elif not Word.match(stuff[0]):
+            res['node'] = stuff[0]
+            stuff = stuff[1:]
+
+        for k in ['job', 'level_of_theory', 'basis', 'formula', 'user', 'date']:
+            res[k] = stuff[0]
+            stuff = stuff[1:]
+
+        res['route'] = subblocks[1]
+        res['description'] = subblocks[2]
+        res['molecule'] = subblocks[3].replace('\\', '\n')
+        res['results'] = []
+        for block in subblocks[4:]:
+            if block == '@': continue
+            for chunk in block.split('\\'):
+                defs = chunk.split('=')
+                if len(defs) == 1:
+                    v = defs[0]
+                    if len(v.strip()) == 0: continue
+                    if numblock_pattern.fullmatch(v):
+                        v = np.array(v.split(',')).astype(float)
+                    res['results'].append(v)
+                elif len(defs) == 2:
+                    v = defs[1]
+                    if ",s," in v:
+                        v = v.split(",")
+                        v = scan_spec(float(v[0]), int(v[2]), float(v[3]))
+                    elif num_pattern.fullmatch(v):
+                        v = float(v)
+                    elif numblock_pattern.fullmatch(v):
+                        v = np.array(v.split(',')).astype(float)
+                    res[defs[0]] = v
+                elif len(chunk.strip()) > 0:
+                    res['results'].append(chunk)
+
+        if res['job'] == 'Freq':
+            if len(res['results']) > 2:
+                fc_scaling = 6 * len(res['results'][0])
+                cq_scaling = len(res['results'][2])
+                if cq_scaling % fc_scaling == 0:
+                    fcs, grad, higher = res['results'][:3]
+                    res['results'] = res['results'][3:]
+                    res['PotentialDeriv'] = [
+                        grad,
+                        fcs,
+                        higher
+                    ]
+                else:
+                    fcs, grad = res['results'][:2]
+                    res['results'] = res['results'][2:]
+                    res['PotentialDeriv'] = [
+                        grad, fcs
+                    ]
+            else:
+                fcs, grad = res['results'][:2]
+                res['results'] = res['results'][2:]
+                res['PotentialDeriv'] = [
+                    grad, fcs
+                ]
+
+        bits.append(res)
+
+    return bits
+
+def tag_validator(block):
+    return "\n\n" not in block
+
+mode = "List"
+GaussianLogComponents["Reports"] = {
+    "tag_start": tag_start,
+    "tag_end": tag_end,
+    'validator': tag_validator,
+    "parser": parse_reports,
+    "mode": mode
+}
+
 ########################################################################################################################
 #
 #                                           GaussianLogDefaults
