@@ -18,12 +18,15 @@ class RDMolecule(ExternalMolecule):
 
     def __init__(self, rdconf, charge=None):
         #atoms, coords, bonds):
+        self._rdmol = rdconf.GetOwningMol()
         super().__init__(rdconf)
         self.charge = charge
 
     @property
     def rdmol(self):
-        return self.mol.GetOwningMol()
+        if self._rdmol is None:
+            self._rdmol = self.mol.GetOwningMol()
+        return self._rdmol
     @property
     def atoms(self):
         mol = self.rdmol
@@ -48,11 +51,14 @@ class RDMolecule(ExternalMolecule):
     def copy(self):
         Chem = self.chem_api()
         conf = self.mol
-        new_mol = Chem.Mol(self.rdmol)
+        new_mol = Chem.AddHs(Chem.Mol(self.rdmol), explicitOnly=True)
         new_mol.AddConformer(conf)
         return type(self).from_rdmol(new_mol,
                                      conf_id=conf.GetId(),
-                                     charge=self.charge, sanitize=False, guess_bonds=False)
+                                     charge=self.charge, sanitize=False,
+                                     guess_bonds=False,
+                                     add_implicit_hydrogens=False
+                                     )
     @property
     def charges(self):
         from rdkit.Chem import AllChem
@@ -94,6 +100,20 @@ class RDMolecule(ExternalMolecule):
         return cls(conf, charge=charge)
 
     @classmethod
+    def resolve_bond_type(cls, t):
+        Chem = cls.chem_api()
+
+        if abs(t - 1.5) < 1e-2:
+            t = Chem.BondType.names["AROMATIC"]
+        elif abs(t - 2.5) < 1e-2:
+            t = Chem.BondType.names["TWOANDAHALF"]
+        elif abs(t - 3.5) < 1e-2:
+            t = Chem.BondType.names["TWOANDAHALF"]
+        else:
+            t = Chem.BondType.values[int(t)]
+
+        return t
+    @classmethod
     def from_coords(cls, atoms, coords, bonds=None, charge=None, guess_bonds=None):
         Chem = cls.chem_api()
         mol = Chem.EditableMol(Chem.Mol())
@@ -109,18 +129,18 @@ class RDMolecule(ExternalMolecule):
                 else:
                     i,j,t = b
                 if nput.is_numeric(t):
-                    t = Chem.BondType.values[int(t)]
+                    t = cls.resolve_bond_type(t)
                 else:
                     t = Chem.BondType.names[t]
                 mol.AddBond(int(i), int(j), t)
         mol.CommitBatchEdit()
 
         mol = mol.GetMol()
+        mol = Chem.AddHs(mol, explicitOnly=True)
         conf = Chem.Conformer(len(atoms))
         conf.SetPositions(np.asanyarray(coords))
         conf.SetId(0)
         mol.AddConformer(conf)
-        mol = Chem.AddHs(mol, explicitOnly=True)
 
         if guess_bonds is None:
             guess_bonds = bonds is None
@@ -163,8 +183,7 @@ class RDMolecule(ExternalMolecule):
         params.useBasicKnowledge = True
         return params
     @classmethod
-    def from_smiles(cls, smiles, num_confs=1, optimize=False, take_min=True,
-                    force_field_type='mmff',
+    def from_smiles(cls, smiles,
                     sanitize=False,
                     parse_name=True,
                     allow_cxsmiles=True,
@@ -173,13 +192,14 @@ class RDMolecule(ExternalMolecule):
                     replacements=None,
                     add_implicit_hydrogens=False,
                     call_add_hydrogens=True,
+                    num_confs=1, optimize=False, take_min=True,
+                    force_field_type='mmff',
                     **opts):
 
         if os.path.isfile(smiles):
             with open(smiles) as f:
                 smiles = f.read()
         Chem = cls.chem_api()
-        AllChem = cls.allchem_api()
 
         params = Chem.SmilesParserParams()
         params.removeHs = remove_hydrogens
@@ -200,9 +220,53 @@ class RDMolecule(ExternalMolecule):
         else:
             mol = rdkit_mol
 
+        return cls.from_base_mol(mol,
+                                 num_confs=1, optimize=False, take_min=True,
+                                 force_field_type='mmff'
+                                 )
+
         # rdDistGeom = RDKitInterface.submodule("Chem.rdDistGeom")
         # rdDistGeom.EmbedMolecule(mol, num_confs, **cls.get_confgen_opts())
 
+    @classmethod
+    def from_base_mol(cls, mol,
+                           conf_id=-1,
+                           num_confs=1,
+                           optimize=False,
+                           take_min=None,
+                           force_field_type='mmff',
+                           **mol_opts):
+        try:
+            conf = mol.GetConformer(conf_id)
+        except ValueError:
+            conf = None
+        if conf:
+            return cls.from_rdmol(mol, conf_id, **mol_opts)
+        else:
+            if conf_id > num_confs + 1:
+                num_confs = conf_id
+            return cls.from_no_conformer_molecule(mol,
+                                                  num_confs=num_confs,
+                                                  optimize=optimize,
+                                                  take_min=conf_id < 0 if take_min is None else take_min,
+                                                  force_field_type=force_field_type,
+                                                  **mol_opts
+                                                  )
+
+    @classmethod
+    def from_no_conformer_molecule(cls,
+                                   mol,
+                                   num_confs=1,
+                                   optimize=False,
+                                   take_min=True,
+                                   force_field_type='mmff',
+                                   add_implicit_hydrogens=False,
+                                   **etc
+                                   ):
+        AllChem = cls.allchem_api()
+
+        AllChem.AddHs(mol, explicitOnly=not add_implicit_hydrogens)
+
         with OutputRedirect():
             conformer_set = AllChem.EmbedMultipleConfs(mol, numConfs=num_confs, params=cls.get_confgen_opts())
         if optimize:
@@ -236,18 +300,23 @@ class RDMolecule(ExternalMolecule):
         else:
             conf_id = 0
 
-        return cls.from_rdmol(mol, conf_id=conf_id)
+        return cls.from_rdmol(mol, conf_id=conf_id, add_implicit_hydrogens=add_implicit_hydrogens, **etc)
+
     def to_smiles(self):
         return self.chem_api().MolToSmiles(self.rdmol)
 
     @classmethod
-    def from_molblock(cls, molblock):
+    def from_molblock(cls,
+                      molblock,
+                      add_implicit_hydrogens=False,
+                      call_add_hydrogens=True,
+                      ):
         Chem = cls.chem_api()
         if os.path.isfile(molblock):
             mol = Chem.MolFromMolFile(molblock, sanitize=False, removeHs=False)
         else:
             mol = Chem.MolFromMolBlock(molblock, sanitize=False, removeHs=False)
-        return cls.from_rdmol(mol)
+        return cls.from_rdmol(mol, add_implicit_hydrogens=add_implicit_hydrogens)
 
     @classmethod
     def allchem_api(cls):
@@ -266,12 +335,24 @@ class RDMolecule(ExternalMolecule):
 
         return ff_type
 
-    def get_force_field(self, force_field_type='mmff', mol=None, conf_id=None, **extra_props):
-        if mol is None:
-            mol = self
-        if conf_id is None:
-            conf_id = mol.mol.GetId()
-        mol = mol.rdmol
+    def get_force_field(self, force_field_type='mmff', conf=None, mol=None, conf_id=None, **extra_props):
+        if conf is None:
+            if mol is None:
+                mol = self
+            if conf_id is None:
+                conf_id = mol.mol.GetId()
+            mol = mol.rdmol
+        else:
+            if mol is None:
+                mol = conf.GetOwningMol()
+            if conf_id is None:
+                conf_id = conf.GetId()
+            if np.sum(np.abs(conf.GetPositions() - mol.GetConformer(conf_id).GetPositions())) > 1e-6:
+                print(mol)
+                print(self.rdmol)
+                raise ValueError(
+                    conf, mol.GetConformer(conf_id)
+                )
 
         force_field_type = self.get_force_field_type(force_field_type)
         if isinstance(force_field_type, (list, tuple)):
@@ -301,6 +382,12 @@ class RDMolecule(ExternalMolecule):
             raise ValueError(f"charge model {model} not supported in RDKit")
 
     def calculate_energy(self, geoms=None, force_field_generator=None, force_field_type='mmff', conf_id=None):
+        Chem = self.chem_api()
+        if conf_id is None:
+            conf_id = self.mol.GetId()
+        mol = Chem.Mol(self.rdmol, confId=conf_id)
+        conf = mol.GetConformer(0)
+
         if force_field_generator is None:
             force_field_generator = self.get_force_field
         if geoms is not None:
@@ -311,11 +398,11 @@ class RDMolecule(ExternalMolecule):
             vals = np.empty(len(geoms), dtype=float)
             try:
                 for i,g in enumerate(geoms):
-                    self.mol.SetPositions(g.copy())
-                    ff = force_field_generator(force_field_type)
+                    conf.SetPositions(g.copy())
+                    ff = force_field_generator(force_field_type, conf=conf, mol=mol)
                     vals[i] = ff.CalcEnergy()
             finally:
-                self.mol.SetPositions(cur_geom)
+                conf.SetPositions(cur_geom)
             return vals.reshape(base_shape)
         else:
             ff = force_field_generator(force_field_type, conf_id=conf_id)
@@ -324,7 +411,14 @@ class RDMolecule(ExternalMolecule):
     def calculate_gradient(self, geoms=None, force_field_generator=None, force_field_type='mmff', conf_id=None):
         if force_field_generator is None:
             force_field_generator = self.get_force_field
-        cur_geom = np.array(self.mol.GetPositions()).reshape(-1, 3)
+
+        Chem = self.chem_api()
+        if conf_id is None:
+            conf_id = self.mol.GetId()
+        mol = Chem.Mol(self.rdmol, confId=conf_id)
+        conf = mol.GetConformer(0)
+
+        cur_geom = np.array(conf.GetPositions()).reshape(-1, 3)
         if geoms is not None:
             geoms = np.asanyarray(geoms)
             base_shape = geoms.shape[:-2]
@@ -332,14 +426,15 @@ class RDMolecule(ExternalMolecule):
             vals = np.empty((len(geoms), np.prod(cur_geom.shape, dtype=int)), dtype=float)
             try:
                 for i, g in enumerate(geoms):
-                    self.mol.SetPositions(g.copy())
-                    ff = force_field_generator(force_field_type)
+                    new_geom = g.copy().view(np.ndarray)
+                    conf.SetPositions(new_geom)
+                    ff = force_field_generator(force_field_type, conf=conf, mol=mol)
                     vals[i] = ff.CalcGrad()
             finally:
-                self.mol.SetPositions(cur_geom)
+                conf.SetPositions(cur_geom)
             return vals.reshape(base_shape + (-1,))
         else:
-            ff = force_field_generator(force_field_type, conf_id=conf_id)
+            ff = force_field_generator(force_field_type, conf_id=conf_id, conf=conf, mol=mol)
             return np.array(ff.CalcGrad()).reshape(-1)
 
     def calculate_hessian(self, force_field_generator=None, force_field_type='mmff', stencil=5, mesh_spacing=.01, **fd_opts):
