@@ -14,6 +14,8 @@ from .ChemToolkits import *; from .ChemToolkits import __all__ as exposed
 __all__ += exposed
 from .ImageKits import *; from .ImageKits import __all__ as exposed
 __all__ += exposed
+from .Toolkits3D import *; from .Toolkits3D import __all__ as exposed
+__all__ += exposed
 from .Visualizers import *; from .Visualizers import __all__ as exposed
 __all__ += exposed
 from .RDKit import *; from .RDKit import __all__ as exposed
@@ -25,4 +27,8 @@ __all__ += exposed
 from .WebAPI import *; from .WebAPI import __all__ as exposed
 __all__ += exposed
 from .ChemSpiderAPI import *; from .ChemSpiderAPI import __all__ as exposed
+__all__ += exposed
+from .Subprocesses import *; from .Subprocesses import __all__ as exposed
+__all__ += exposed
+from .Servers import *; from .Servers import __all__ as exposed
 __all__ += exposed
\ No newline at end of file
