@@ -1,5 +1,8 @@
 
 import itertools, collections
+import typing
+
+import scipy
 import scipy.sparse as sparse, numpy as np
 from .. import Numputils as nput
 from .. import Iterators as itut
@@ -11,7 +14,8 @@ __all__ = [
 ]
 
 class EdgeGraph:
-    __slots__ = ["labels", "edges", "graph", "map"]
+    map: dict[int, set[int]]
+    __slots__ = ["labels", "edges", "graph", "map", "_rings", "_sp_data"]
     def __init__(self, labels, edges, graph=None, edge_map=None):
         self.labels = labels
         self.edges = np.asanyarray(edges)
@@ -22,13 +26,68 @@ class EdgeGraph:
             edge_map = self.build_edge_map(self.edges)
         self.map = edge_map
         self._rings = None
+        self._sp_data = None
+
+    def graph_difference(self, other):
+        diff = (other.graph - self.graph)
+        if isinstance(diff, np.ndarray):
+            nzr, nzc = np.nonzero(diff)
+            new = diff[nzr, nzc].asarray() > 0
+        else:
+            nzr, nzc, vals = sparse.find(diff)
+            new = vals > 0
+        old = np.logical_not(new)
+
+        return np.array([nzr[new], nzc[new]]).T, np.array([nzr[old], nzc[old]]).T
+
+    @classmethod
+    def get_edge_graph(cls, spec, num_nodes=None):
+        if isinstance(spec, dict):
+            edge_list = np.array([
+                (i, j)
+                for i in spec
+                for j in spec[i]
+            ])
+            if num_nodes is None:
+                num_nodes = len(np.unique(edge_list.flatten()))
+            return cls.adj_mat(num_nodes, edge_list)
+        elif isinstance(spec, (sparse.csr_matrix, sparse.coo_matrix, sparse.csc_matrix)): #TODO: better checks
+            return spec
+        else:
+            spec = np.array(spec)
+            num_nodes = len(np.unique(spec.flatten()))
+            return cls.adj_mat(num_nodes, spec)
+
+    @classmethod
+    def get_edge_list(cls, spec):
+        if isinstance(spec, dict):
+            return [
+                (i, j)
+                for i in spec
+                for j in spec[i]
+            ]
+        elif isinstance(spec, (sparse.csr_matrix, sparse.coo_matrix, sparse.csc_matrix)): #TODO: better checks
+            r, c = spec.nonzero()
+            return np.array([r, c]).T
+        else:
+            return spec
+
+    @classmethod
+    def get_edge_map(cls, spec):
+        if isinstance(spec, dict):
+            return spec
+        elif isinstance(spec, (sparse.csr_matrix, sparse.coo_matrix, sparse.csc_matrix)):  # TODO: better checks
+            return cls.get_edge_map(cls.get_edge_list(spec))
+        else:
+            return cls.build_edge_map(spec)
 
     @classmethod
     def adj_mat(cls, num_nodes, edges):
-        adj = np.zeros((num_nodes, num_nodes), dtype=bool)
-        rows,cols = edges.T
-        adj[rows, cols] = 1
-        adj[cols, rows] = 1
+        adj = np.zeros((num_nodes, num_nodes), dtype=int)
+        if len(edges) > 0:
+            rows,cols = edges.T
+            adj[rows, cols] = 1
+            adj[cols, rows] = 1
 
         return sparse.csr_matrix(adj)
 
@@ -41,13 +100,16 @@ class EdgeGraph:
         )
 
     @classmethod
-    def build_edge_map(cls, edge_list):
+    def build_edge_map(cls, edge_list, num_nodes=None):
         map = {}
         for e1,e2 in edge_list:
             if e1 not in map: map[e1] = set()
             map[e1].add(e2)
             if e2 not in map: map[e2] = set()
             map[e2].add(e1)
+        if num_nodes is not None:
+            for i in range(num_nodes):
+                if i not in map: map[i] = set()
         return map
 
     @classmethod
@@ -63,7 +125,7 @@ class EdgeGraph:
 
         return [labels[p] for p in pos], edge_list
     @classmethod
-    def _take(cls, pos, labels, adj_mat:sparse.compressed):
+    def _take(cls, pos, labels, adj_mat:sparse.compressed) -> 'typing.Self':
         rows, cols, _ = sparse.find(adj_mat)
         utri = cols >= rows
         rows = rows[utri]
@@ -179,22 +241,23 @@ class EdgeGraph:
         if self._rings is None:
             self._rings = self.get_rings()
         return self._rings
-    def get_rings(self):
-        # clunky, but it works
+
+    @classmethod
+    def find_rings_in_graph(cls, n_inds, edge_map):
         test_rings = []
-        for n,l in enumerate(self.labels):
-            if len(self.map[n]) < 2: continue
+        for n in range(n_inds):
+            if len(edge_map[n]) < 2: continue
             visited = set()
             test_rings.append(
                 tree.tree_traversal(
-                    self.map,
+                    edge_map,
                     lambda parent, child, visited: (
                         visited
-                            if len(visited & self.map[child]) > 1 else
+                        if len(visited & edge_map[child]) > 1 else
                         None
                     ),
                     root=n,
-                    get_children=lambda h: [c for c in self.map[h] if len(self.map[c]) > 1],
+                    get_children=lambda h: [c for c in edge_map[h] if len(edge_map[c]) > 1],
                     get_item=lambda _, h: h,
                     visited=visited,
                     call_order='post',
@@ -206,45 +269,289 @@ class EdgeGraph:
 
         rings = []
         keys = set()
-        for i,r1 in enumerate(filt_rings):
+        for i, r1 in enumerate(filt_rings):
             for r2 in filt_rings[i:]:
                 int_ring = r1 & r2
                 key = tuple(sorted(int_ring))
                 if key in keys: continue
                 keys.add(key)
-                r = self.check_ring(int_ring)
+                r = cls.check_ring_in_graph(int_ring, edge_map)
                 if r: rings.append(r)
         return rings
 
-
-    def check_ring(self, ring_atoms):
+    @classmethod
+    def check_ring_in_graph(cls, ring_atoms, edge_map):
         if len(ring_atoms) < 3: return False
         ring_atoms = set(ring_atoms)
-        if any(len(self.map[r] & ring_atoms) == 1 for r in ring_atoms): return False
+        if any(len(edge_map[r] & ring_atoms) == 1 for r in ring_atoms): return False
         visited = set()
         root = next(iter(ring_atoms))
         ring = []
         return tree.tree_traversal(
-                    self.map,
+                    edge_map,
                     lambda parent, child, visited: (
                         ring + [child]
-                            if (parent != root and root in self.map[child]) else
+                            if (parent != root and root in edge_map[child]) else
                         ring.append(child)
                     ),
                     root=root,
-                    get_children=lambda h: self.map[h] & ring_atoms,
+                    get_children=lambda h: edge_map[h] & ring_atoms,
                     get_item=lambda _, h: h,
                     visited=visited,
                     call_order='post',
                     traversal_ordering='dfs'
                 )
 
-    def get_fragments(self):
-        from scipy.sparse import csgraph
-        _, labels = csgraph.connected_components(self.graph, directed=False, return_labels=True)
+    def get_rings(self):
+        return self.find_rings_in_graph(len(self.labels), self.map)
+
+    @classmethod
+    def get_shortest_path_data(cls, graph):
+        return sparse.csgraph.shortest_path(
+            graph, directed=False, return_predecessors=True
+        )
+    @property
+    def shortest_path_data(self):
+        if self._sp_data is None:
+            self._sp_data = self.get_shortest_path_data(self.graph)
+        return self._sp_data
+
+    @classmethod
+    def get_path_from_data(cls, start, end, sp_data):
+        dist_matrix, predecessors = sp_data
+        path = [end]
+        # straight from scipy docs...I had a better imp. at one point but dunno where it is
+        for _ in range(len(dist_matrix)):  # max number of steps
+            cur_node = int(predecessors[start][path[-1]])
+            if cur_node < 0:
+                break
+            path.append(cur_node)
+        return tuple(reversed(path))
+    @classmethod
+    def get_longest_path_from_data(cls, shortest_path_data, root=None):
+        dist_matrix, predecessors = shortest_path_data
+        if root is None:
+            max_cols = np.argmax(dist_matrix, axis=1)
+            max_row = np.argmax(dist_matrix[np.arange(len(max_cols)), max_cols])
+            return cls.get_path_from_data(max_row, max_cols[max_row], shortest_path_data)
+        else:
+            max_col = np.argmax(dist_matrix[root], axis=0)
+            return cls.get_path_from_data(root, max_col, shortest_path_data)
+    def get_path(self, start, end):
+        return self.get_path_from_data(start, end, self.shortest_path_data)
+
+    @classmethod
+    def compute_edge_centralities(self, indices, map):
+        if nput.is_int(indices):
+            return len(map[indices])
+        else:
+            return np.array([len(map[i]) for i in indices])
+
+    @classmethod
+    def find_longest_chain_from_breakpoints(cls,
+                                            map,
+                                            graph=None,
+                                            rings=None,
+                                            root=None,
+                                            use_highest_valencies=True,
+                                            shortest_path_data=None
+                                            ):
+        if rings is None:
+            rings = cls.find_rings_in_graph(len(map), map)
+        if graph is None:
+            graph = cls.get_edge_graph(map)
+        if len(rings) == 0:
+            if shortest_path_data is None:
+                shortest_path_data = cls.get_shortest_path_data(graph)
+            return cls.get_longest_path_from_data(shortest_path_data, root=root)
+        else:
+            # in principle we have to check every set of breakpoints...
+            # but we can make this more efficient by noting that a break
+            # has to occur near the highest-valency position for fused rings
+            # and if breaking can lead to a long ring will again be near a high-valency
+            # atom in non-fused rings
+            # each potential high-valency bond can be broken
+            if use_highest_valencies:
+                ring_valencies = [cls.compute_edge_centralities(r, map) for r in rings]
+                break_pos = [
+                    np.array(r)[np.where(v == np.max(v))]
+                    for r,v in zip(rings, ring_valencies)
+                ]
+            else:
+                break_pos = rings
+
+            break_bonds = [
+                [
+                    (p, j)  # enumerate all ring bonds starting at the breakpos, could be more efficient
+                    for p in bp
+                    for j in map[p]
+                    if j in r
+                ]
+                for r, bp in zip(rings, break_pos)
+            ]
+            if root is not None:
+                # sort break bonds to priortize breaks by the root
+                break_bonds = [
+                    list(sorted(bl, key=lambda p: p[0] != root and p[1] != 1))
+                    for bl in break_bonds
+                ]
+
+            # chain = None
+            n_inds = len(map)
+            for p_list in itertools.product(*break_bonds):
+                if len({tuple(sorted(p)) for p in p_list}) < len(p_list): continue # dupe bond in fused ring
+                new_map: dict[int, set[int]] = {k: v.copy() for k, v in map.items()}
+                for center, bond in p_list:
+                    new_map[center].remove(bond)
+                    new_map[bond].remove(center)
+
+                new_rings = cls.find_rings_in_graph(n_inds, new_map)
+                if len(new_rings) == 0:
+                    chain = cls.find_longest_chain_from_breakpoints(
+                        new_map,
+                        root=root
+                    )
+                    break
+            else:
+                raise ValueError("couldn't find a way to break rings?")
+
+            return chain
+
+    def find_longest_chain(self,
+                           rings=None,
+                           use_highest_valencies=True,
+                           ):
+        if rings is None:
+            rings = self.rings
+
+        return self.find_longest_chain_from_breakpoints(
+            self.map,
+            graph=self.graph,
+            rings=rings,
+            use_highest_valencies=use_highest_valencies,
+            shortest_path_data=self.shortest_path_data
+        )
+
+    def segment_by_chains(self,
+                          rings=None,
+                          root=None,
+                          use_highest_valencies=True,
+                          ):
+        return self.segment_graph_by_chains(
+            self.map,
+            root=root,
+            graph=self.graph,
+            rings=self.rings,
+            use_highest_valencies=use_highest_valencies,
+            shortest_path_data=self.shortest_path_data
+        )
+
+    @classmethod
+    def find_graph_centroid(cls, graph, shortest_path_data=None):
+        # we define the graph centroid as the point with the minimum maximum graph
+        # distance over a fully-connected graph
+
+        if shortest_path_data is None:
+            shortest_path_data = cls.get_shortest_path_data(graph)
+        distance_matrix, _ = shortest_path_data
+        return np.argmin(np.argmax(distance_matrix, axis=1), axis=0)
+
+    def get_centroid(self, check_fragments=True):
+        if check_fragments:
+            fragments = self.get_fragments()
+            if len(fragments) > 0:
+                centroids = []
+                for f in fragments:
+                    g = self.take(f)
+                    centroids.append(g.get_centroid(check_fragments=False))
+                return fragments, centroids
+
+        return self.find_graph_centroid(self.graph, shortest_path_data=self.shortest_path_data)
+
+    @classmethod
+    def get_graph_fragment_indices(cls, graph):
+        _, labels = sparse.csgraph.connected_components(graph, directed=False, return_labels=True)
         _, groups = nput.group_by(np.arange(len(labels)), labels)[0]
         return groups
 
+    def get_fragments(self):
+        return self.get_graph_fragment_indices(self.graph)
+
+    @classmethod
+    def _reindex_segmentss(cls, frags, remapping):
+        if nput.is_int(frags[0]):
+            return tuple(remapping[j] for j in frags)
+        else:
+            return tuple(
+                cls._reindex_segmentss(f, remapping)
+                for f in frags
+            )
+    @classmethod
+    def segment_graph_by_chains(cls,
+                                map: dict[int, set[int]],
+                                graph: ('sparse.coo_matrix|sparse.csr_matrix|sparse.csc_matrix') = None,
+                                rings=None,
+                                root=None,
+                                use_highest_valencies=True,
+                                shortest_path_data=None
+                                ):
+
+        if len(map) == 1:
+            return tuple(map.keys())
+
+        if graph is None:
+            graph = cls.get_edge_graph(map)
+
+        segments = []
+
+        backbone = cls.find_longest_chain_from_breakpoints(
+            map,
+            graph=graph,
+            rings=rings,
+            root=root,
+            use_highest_valencies=use_highest_valencies,
+            shortest_path_data=shortest_path_data
+        )
+
+        segments.append(backbone)
+
+        new_map = {k:v.copy() for k,v in map.items()}
+        for i in backbone:
+            for j in new_map.get(i, set()):
+                new_map[j].remove(i)
+            del new_map[i]
+
+        rem = np.delete(np.arange(graph.shape[0]), backbone)
+        base_remapping = {k: i for i, k in enumerate(rem)}
+        base_inv = {i: k for i, k in enumerate(rem)}
+        new_map = {
+            base_remapping[i]: {base_remapping[k] for k in v}
+            for i,v in new_map.items()
+        }
+
+        fragments = cls.get_graph_fragment_indices(
+            cls.get_edge_graph(new_map, num_nodes=len(rem))
+        )
+
+        for frag in fragments:
+            if len(frag) == 1:
+                segments.append((tuple(frag),))
+            else:
+                remapping = {k:i for i,k in enumerate(frag)}
+                inverse_mapping = {i:base_inv[k] for i,k in enumerate(frag)}
+                subbones = cls.segment_graph_by_chains(
+                    {
+                        remapping[k]: {remapping[j] for j in new_map[k]}
+                        for k in frag
+                    },
+                    use_highest_valencies=use_highest_valencies
+                )
+                reind = cls._reindex_segmentss(subbones, inverse_mapping)
+                segments.append(reind)
+
+        return segments
+
+
     @classmethod
     def get_maximum_overlap_permutation(cls, graph_1:'EdgeGraph', graph_2:'EdgeGraph'):
         syms_1 = graph_1.labels
@@ -669,4 +976,70 @@ class MoleculeEdgeGraph(EdgeGraph):
             for n, _ in enumerate(self.labels)
         ]
 
+    light_atoms = {"H", "D"}
+    def get_heavy_atom_framework_graph(
+            self,
+            heavy_atoms=None,
+            light_atoms=None,
+            included_atoms=None
+    ):
+        if heavy_atoms is None:
+            if light_atoms is None:
+                light_atoms = self.light_atoms
 
+            inds = [
+                i for i,l in enumerate(self.labels)
+                if l not in light_atoms
+            ]
+        else:
+            inds = [
+                i for i,l in enumerate(self.labels)
+                if l in heavy_atoms
+            ]
+        if included_atoms is not None:
+            inds = [i for i in inds if i not in inds] + inds
+
+        return self.take(inds)
+    def find_longest_chain(self,
+                           rings=None,
+                           root=None,
+                           use_highest_valencies=True,
+                           heavy_atoms=True,
+                           light_atoms=None
+                           ):
+        if heavy_atoms or (light_atoms is not None):
+            if heavy_atoms is True: heavy_atoms = None
+            graph = self.get_heavy_atom_framework_graph(
+                included_atoms=[root] if root is not None else None,
+                heavy_atoms=heavy_atoms,
+                light_atoms=light_atoms
+            )
+            return graph.find_longest_chain(
+                root=0 if root is not None else None,
+                heavy_atoms=False,
+                light_atoms=None
+            )
+        else:
+            return super().find_longest_chain()
+
+    def segment_by_chains(self,
+                          root=None,
+                          rings=None,
+                          use_highest_valencies=True,
+                          heavy_atoms=True,
+                          light_atoms=None
+                          ):
+        if heavy_atoms or (light_atoms is not None):
+            if heavy_atoms is True: heavy_atoms = None
+            graph = self.get_heavy_atom_framework_graph(
+                included_atoms=[root] if root is not None else None,
+                heavy_atoms=heavy_atoms,
+                light_atoms=light_atoms
+            )
+            return graph.segment_by_chains(
+                root=0 if root is not None else None,
+                heavy_atoms=False,
+                light_atoms=None
+            )
+        else:
+            return super().segment_by_chains(root=root)
