@@ -1,6 +1,8 @@
 """
 A module of useful math for handling coordinate transformations and things
 """
+from __future__ import annotations
+import enum
 import itertools
 import math
 
@@ -22,6 +24,7 @@ __all__ = [
     "block_array",
     "vec_tdot",
     "distance_matrix",
+    "points_from_distance_matrix",
     "identity_tensors",
     "semisparse_tensordot",
     "frac_powh",
@@ -58,7 +61,8 @@ __all__ = [
     "symmetric_matrix_log",
     "imaginary_symmetric_matrix_log",
     "sylvester_solve",
-    "symmetrize_array"
+    "symmetrize_array",
+    "integer_exponent"
 ]
 
 ##
@@ -1188,6 +1192,7 @@ def polar_to_cartesian(center, v, u, r, a, d):
         newstuff = affine_multiply(transfs, r[..., np.newaxis] * vecs1)
     return newstuff
 
+
 ##############################################################################
 #
 #       apply_pointwise
@@ -1358,21 +1363,26 @@ def find_basis(mat, nonzero_cutoff=1e-8, method='svd'):
     else:
         raise ValueError(f"method `{method}`")
 
-def projection_matrix(basis, orthonormal=False):
+def projection_matrix(basis, inverse=None, orthonormal=False, allow_pinv=False):
     basis = np.asanyarray(basis)
     if basis.ndim == 1:
         basis = basis[np.newaxis]
-    if not orthonormal:
-        basis, _ = np.linalg.qr(basis)
+    if inverse is None:
+        if allow_pinv:
+            inverse = np.linalg.pinv(basis)
+        else:
+            if not orthonormal:
+                basis, _ = np.linalg.qr(basis)
+            inverse = np.moveaxis(basis, -2, -1)
 
-    return basis @ np.moveaxis(basis, -2, -1)
+    return basis @ inverse
 
-def orthogonal_projection_matrix(basis, orthonormal=False):
-    proj = projection_matrix(basis, orthonormal=orthonormal)
+def orthogonal_projection_matrix(basis, inverse=None, orthonormal=False, allow_pinv=False):
+    proj = projection_matrix(basis, inverse=inverse, orthonormal=orthonormal, allow_pinv=allow_pinv)
     identities = identity_tensors(proj.shape[:-2], proj.shape[-1])
     return identities - proj
 
-def _proj(projection_type, vecs, basis, ndim=None, orthonormal=False):
+def _proj(projection_type, vecs, basis, ndim=None, orthonormal=False, inverse=None, allow_pinv=False):
     vecs = np.asanyarray(vecs)
     basis = np.asanyarray(basis)
     if vecs.shape[-1] != basis.shape[-2]:
@@ -1385,7 +1395,7 @@ def _proj(projection_type, vecs, basis, ndim=None, orthonormal=False):
             ndim = vecs.ndim - len(base_shape)
     base_shape = vecs.shape[:-ndim]
 
-    proj = projection_type(basis, orthonormal=orthonormal)
+    proj = projection_type(basis, orthonormal=orthonormal, inverse=inverse, allow_pinv=allow_pinv)
     if ndim == 1:
         vecs = (vecs[..., np.newaxis, :] @ proj).reshape(vecs.shape)
     else:
@@ -1394,11 +1404,13 @@ def _proj(projection_type, vecs, basis, ndim=None, orthonormal=False):
 
     return vecs
 
-def project_onto(vecs, basis, ndim=None, orthonormal=False):
-    return _proj(projection_matrix, vecs, basis, ndim=ndim, orthonormal=orthonormal)
+def project_onto(vecs, basis, ndim=None, orthonormal=False, inverse=None, allow_pinv=False):
+    return _proj(projection_matrix, vecs, basis, ndim=ndim,
+                 orthonormal=orthonormal, inverse=inverse, allow_pinv=allow_pinv)
 
-def project_out(vecs, basis, ndim=None, orthonormal=False):
-    return _proj(orthogonal_projection_matrix, vecs, basis, ndim=ndim, orthonormal=orthonormal)
+def project_out(vecs, basis, ndim=None, orthonormal=False, inverse=None, allow_pinv=False):
+    return _proj(orthogonal_projection_matrix, vecs, basis, ndim=ndim,
+                 orthonormal=orthonormal, inverse=inverse, allow_pinv=allow_pinv)
 
 def fractional_power(A, pow, zero_cutoff=1e-8):
     # only applies to symmetric A
@@ -1641,4 +1653,28 @@ def symmetrize_array(a,
                 new_pos = tuple(new_pos[i] for i in ordering)
                 b[new_pos] = val
 
-    return b
\ No newline at end of file
+    return b
+
+def integer_exponent(ints, k, max_its=None):
+    ints = np.asanyarray(ints, dtype=int)
+    base_shape = ints.shape
+    ints = ints.reshape(-1)
+    sel = np.arange(ints.shape[0])
+    counts = np.zeros(ints.shape[0], dtype=int)
+    if max_its is None:
+        if k == 2:
+            test_its = np.log2(ints)
+        elif k == 10:
+            test_its = np.log10(ints)
+        else:
+            test_its = np.log(ints) / np.log(k)
+        max_its = int(np.ceil(np.max(test_its)))
+    for i in range(max_its):
+        mask = np.where(ints[sel,] % k == 0)
+        if len(mask) == 0 or len(mask[0]) == 0:
+            break
+
+        sel = sel[mask[0],]
+        counts[sel,] += 1
+        ints[sel,] //= k
+    return ints.reshape(base_shape), counts.reshape(base_shape)
\ No newline at end of file
