@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import enum
 from mmap import mmap
 import abc, io
@@ -314,7 +316,8 @@ class SearchStreamReader:
     def find_tag(self,
                  tag,
                  skip_tag=None,
-                 seek=None
+                 seek=None,
+                 allow_terminal=False
                  ):
         """
         Finds a tag in a file
@@ -352,7 +355,7 @@ class SearchStreamReader:
                 for tag in follow_ups:
                     self.stream.seek(pos + 1)
                     p = self.find_tag(tag)
-                    if p > -1:
+                    if allow_terminal or p > -1:
                         pos = p
 
             offset = tags.offset
@@ -361,10 +364,13 @@ class SearchStreamReader:
                 # I won't touch it for now but I feel like it should be pos
                 pos = self.stream.tell() + offset
                 self.stream.seek(pos)
+            break # first match in alternatives
 
         return pos
 
-    def get_tagged_block(self, tag_start, tag_end, block_size=500):
+    class TagSentinels(enum.Enum):
+        EOF = 'end_of_file'
+    def get_tagged_block(self, tag_start, tag_end, validator=None, allow_terminal=False, block_size=500):
         """
         Pulls the string between tag_start and tag_end
 
@@ -375,26 +381,50 @@ class SearchStreamReader:
         :return:
         :rtype:
         """
-        if tag_start is not None:
-            start = self.find_tag(tag_start)
-            if start >= 0:
-                with FileStreamCheckPoint(self):
-                    end = self.find_tag(tag_end, seek=False)
-                if end > start:
-                    return self.stream.read(end-start)
-        else:
-            start = self.tell()
-            with FileStreamCheckPoint(self):
-                end = self.find_tag(tag_end, seek=False)
-            if end >= start:
-                return self.stream.read(end-start)
 
-        # implict None return if no block found
+        block = None
+        while block is None:
+            if tag_start is not None:
+                    start = self.find_tag(tag_start, allow_terminal=False)
+                    if start >= 0:
+                        with FileStreamCheckPoint(self):
+                            end = self.find_tag(tag_end, allow_terminal=allow_terminal, seek=False)
+                        if end > start:
+                            block = self.stream.read(end-start)
+                        elif allow_terminal and end < 0:
+                            block = self.stream.read()
+                        else:
+                            block = self.TagSentinels.EOF
+                    else:
+                        block = self.TagSentinels.EOF
+            else:
+                start = self.tell()
+                with FileStreamCheckPoint(self):
+                    end = self.find_tag(tag_end, allow_terminal=allow_terminal, seek=False)
+                if end >= start:
+                    block = self.stream.read(end-start)
+                elif allow_terminal and end < 0:
+                    block = self.stream.read()
+                else:
+                    block = self.TagSentinels.EOF
+
+            if (
+                    block is not None
+                    and block is not self.TagSentinels.EOF
+                    and validator is not None
+                    and not validator(block)
+            ):
+                block = None
+
+        if block is self.TagSentinels.EOF:
+            block = None
+        return block
 
     def parse_key_block(self,
                         tag_start=None,
                         tag_end=None,
                         mode="Single",
+                        validator=None,
                         parser=None,
                         parse_mode="List",
                         num=None,
@@ -437,7 +467,7 @@ class SearchStreamReader:
 
                     i = 0 # protective
                     for i in range(num):
-                        block = self.get_tagged_block(tag_start, tag_end)
+                        block = self.get_tagged_block(tag_start, tag_end, validator=validator)
                         if block is None:
                             break
                         if parse_mode != "List":
@@ -451,7 +481,7 @@ class SearchStreamReader:
                         blocks = parser(blocks[:i+1])
                 else:
                     blocks = []
-                    block = self.get_tagged_block(tag_start, tag_end)
+                    block = self.get_tagged_block(tag_start, tag_end, validator=validator)
                     if parser is None:
                         parser = lambda a, reader=None:a
                     while block is not None:
@@ -462,7 +492,7 @@ class SearchStreamReader:
                                 block = parser(block)
 
                         blocks.append(block)
-                        block = self.get_tagged_block(tag_start, tag_end)
+                        block = self.get_tagged_block(tag_start, tag_end, validator=validator)
 
                     if parse_mode == "List":
                         if pass_context:
@@ -471,7 +501,7 @@ class SearchStreamReader:
                             blocks = parser(blocks)
                 return blocks
         else:
-            block = self.get_tagged_block(tag_start, tag_end)
+            block = self.get_tagged_block(tag_start, tag_end, validator=validator)
             if parser is not None:
                 block = parser(block)
             return block
