@@ -22,41 +22,40 @@ from .SceneJSON import SceneJSON as sceneJSON
 
 DPI_SCALING = 72
 
-class XAxisManager:
+class AxisManager:
     def __init__(self,
                  tick_getter,
                  tick_setter,
                  tick_locator,
-                 minor_tick_locator
+                 minor_tick_locator,
+                 tick_formatter,
+                 minor_tick_formatter
                  ):
-        self.get_xticks = tick_getter
-        self.set_xticks = tick_setter
+        self.get_ticks = tick_getter
+        self.set_ticks = tick_setter
         self.set_major_locator = tick_locator
         self.set_minor_locator = minor_tick_locator
+        self.set_major_formatter = tick_formatter
+        self.set_minor_formatter = minor_tick_formatter
 
-class YAxisManager:
-    def __init__(self,
-                 tick_getter,
-                 tick_setter,
-                 tick_locator,
-                 minor_tick_locator
-                 ):
-        self.get_yticks = tick_getter
-        self.set_yticks = tick_setter
-        self.set_major_locator = tick_locator
-        self.set_minor_locator = minor_tick_locator
 
-class ZAxisManager:
-    def __init__(self,
-                 tick_getter,
-                 tick_setter,
-                 tick_locator,
-                 minor_tick_locator
-                 ):
-        self.get_zticks = tick_getter
-        self.set_zticks = tick_setter
-        self.set_major_locator = tick_locator
-        self.set_minor_locator = minor_tick_locator
+class XAxisManager(AxisManager):
+    def get_xticks(self):
+        return self.get_ticks()
+    def set_xticks(self, ticks, **kwargs):
+        return self.set_ticks(ticks, **kwargs)
+
+class YAxisManager(AxisManager):
+    def get_yticks(self):
+        return self.get_ticks()
+    def set_yticks(self, ticks, **kwargs):
+        return self.set_ticks(ticks, **kwargs)
+
+class ZAxisManager(AxisManager):
+    def get_zticks(self):
+        return self.get_ticks()
+    def set_zticks(self, ticks, **kwargs):
+        return self.set_ticks(ticks, **kwargs)
 
 class GraphicsAxes(metaclass=abc.ABCMeta):
     """
@@ -68,12 +67,16 @@ class GraphicsAxes(metaclass=abc.ABCMeta):
             self.get_xticks,
             self.set_xticks,
             None,
+            None,
+            None,
             None
         )
         self.yaxis = YAxisManager(
             self.get_yticks,
             self.set_yticks,
             None,
+            None,
+            None,
             None
         )
 
@@ -222,6 +225,8 @@ class GraphicsAxes3D(GraphicsAxes):
             self.get_zticks,
             self.set_zticks,
             None,
+            None,
+            None,
             None
         )
 
@@ -2218,9 +2223,9 @@ class X3DAxes(GraphicsAxes3D):
                   edgecolors=None, color=None, glow=None, **styles):
         if color is None: color = edgecolors
         if color is None: color = 'black'
-        if line_thickness is None and s is not None:
-            if not nput.is_numeric(s): s = s[0]
-            line_thickness = s / 1000
+        # if line_thickness is None and s is not None:
+        #     if not nput.is_numeric(s): s = s[0]
+        #     line_thickness = s / 1000
         points = np.asanyarray(points)
         if riffle:
             if indices is not None:
@@ -2240,13 +2245,26 @@ class X3DAxes(GraphicsAxes3D):
                     points[..., riff_start, np.newaxis, :],
                     points[..., riff_end, np.newaxis, :]
                 ], axis=-2).reshape((-1, 2, 3))
-        if glow is None:
-            glow = color
 
-        if indices is not None:
-            line_set = x3d.X3DIndexedLineSet(points, indices, line_thickness=line_thickness, glow=glow, **styles)
+        if line_thickness is not None:
+            if indices is not None:
+                raise NotImplementedError(indices)
+            else:
+                # line_set = x3d.X3DGroup([
+                #     x3d.X3DCylinder(p1, p2, line_thickness=line_thickness, color=glow, **styles)
+                #     for p1, p2 in zip(points[:-1], points[1:])
+                # ])
+                line_set = x3d.X3DCylinder(points[:-1], points[1:], radius=line_thickness,
+                                           glow=glow,
+                                           color=color,
+                                           **styles)
         else:
-            line_set = x3d.X3DLine(points, line_thickness=line_thickness, glow=glow, **styles)
+            if glow is None:
+                glow = color
+            if indices is not None:
+                line_set = x3d.X3DIndexedLineSet(points, indices, line_thickness=line_thickness, glow=glow, **styles)
+            else:
+                line_set = x3d.X3DLine(points, line_thickness=line_thickness, glow=glow, **styles)
         self.children.append(line_set)
 
         return line_set
